//---------------------------------------------------------------------------

#include <vcl.h>
#include <iostream>
#include <iterator>
#include <iomanip>
#include <map>
#include <string>
#include <algorithm>
#include <conio.h> //  getch()

#pragma hdrstop

//---------------------------------------------------------------------------

#pragma argsused
using namespace std;

std::string ToRus(const std::string &in)
{
  char *buff = new char [in.length()+1];
  CharToOem(in.c_str(),buff);
  std::string out(buff);
  delete [] buff;
  return out;
}

//     
// -       
// -      
class RuntimeStringCmp { 
  public:
    //   
    enum cmp_mode {normal, nocase}; 
  private:
    //   
    const cmp_mode mode;

    //         
    static bool nocase_compare (char c1, char c2)
    {
      return toupper(c1) < toupper(c2);
    }
  public:
    // :    
    RuntimeStringCmp (cmp_mode m=normal) : mode(m) {}
    
    // 
    bool operator() (const string& s1, const string& s2) const { 
      if (mode == normal) { 
        return s1 < s2;
      } else {
        return lexicographical_compare (s1.begin(), s1.end(),
                                        s2.begin(), s2.end(),
                                        nocase_compare);
     }
    }
};
  
//  :
//  - 
//  - : string
//  - : string
//  -    

typedef map<string,string,RuntimeStringCmp> StringStringMap;

//         
void fillAndPrint(StringStringMap& coll);
 

int main(int argc, char* argv[])
{
  //       
  cout << ToRus(" 1 ( ):\n");
  StringStringMap coll1;
  fillAndPrint(coll1);

  //        
  RuntimeStringCmp ignorecase(RuntimeStringCmp::nocase);

  //        
  cout << ToRus(" 2 ( ):\n");
  StringStringMap coll2(ignorecase);
  fillAndPrint(coll2);

  getch();
  return 0;
}

void fillAndPrint(StringStringMap& coll)
{
  //     
  coll["Deutschland"] = ""; 
  coll["deutsch"] = ""; 
  coll["Haken"] = ""; 
  coll["arbeiten"] = ""; 
  coll["Hund"] = ""; 
  coll["gehen"] = ""; 
  coll["Unternehmen"] = ""; 
  coll["unternehmen"] = ""; 
  coll["gehen"] = ""; 
  coll["Bestatter"] = "";

  //   
  StringStringMap::iterator pos; 
  cout.setf(ios::left,ios::adjustfield); 
  for (pos=coll.begin(); pos!=coll.end(); ++pos) { 
    cout << setw(15) << pos->first.c_str() << " " 
         << ToRus(pos->second) << endl;
  }
  cout << endl;
}

//---------------------------------------------------------------------------
